/****************************************************************************
 * This file is part of the program MusicRater.                             *
 * MusicRater is a program used to rate your music while listening to them  *
 * Copyright (C) 2005, 2006 Michael "Serpedon" Walz   (serpedon@gmx.de)     *
 *                                                                          *
 * This program is free software; you can redistribute it and/or modify     *
 * it under the terms of the GNU General Public License as published by     *
 * the Free Software Foundation; either version 2 of the License, or        *
 * (at your option) any later version.                                      *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License along  *
 * with this program; if not, write to the Free Softwares Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA                 *
 ****************************************************************************/
 
// Schutz vor mehrfachem Einbinden
# ifndef __KLASSE_CREATE_FILE_HASH_INCLUDED__
# define __KLASSE_CREATE_FILE_HASH_INCLUDED__

# include <QObject>
# include <QProcess>
# include <QMutex>

class C_CreateFileHash : public QObject
{
	Q_OBJECT
	
public:
	enum HashType { WholeFile = 0, IgnoreBackID2 = 1, IgnoreFrontID3 = 2 };
	Q_DECLARE_FLAGS ( HashTypeFlag, HashType );
	
	enum FailedReasons { CannotOpenFile, CannotReadFromFile, ProcessDidNotSuccess, CannotWriteToHashCommand, ID3SyncByteNotFound, HashCommandNotSet };

	C_CreateFileHash ( const QString & f_filename, 
		/* <-- */		const bool f_selfDestoryAfterWork = false, 
		/* <-- */		const C_CreateFileHash::HashTypeFlag f_hashType = C_CreateFileHash::WholeFile );
	
public slots:
	void Start ( );

signals:
	void HashCode ( QString f_hash, QString f_filename, C_CreateFileHash::HashTypeFlag f_hashType );
	void HashingFailed ( C_CreateFileHash::FailedReasons );

private slots:
	void ProcessFinished ( int f_exitCode );
	
private:
	void SelfDestroy ( );
	void ErrorHashingFailed ( C_CreateFileHash::FailedReasons f_failedReason );

	const QString m_filename;
	const bool m_selfDestoryAfterWork;
	const C_CreateFileHash::HashTypeFlag m_hashType;
	QProcess	m_hashProcess;
	
/*static*/ private:
	static const	qint64 m_bufferSize = 1024 * 64;
	static const	qint64 m_sizeOfID2Tag = 128;
	static const	qint64 m_maxAmountOfSyncProcesses = 1;
	static			qint64 m_curAmountOfSyncProcesses;
	static QMutex m_mutex_amountOfSyncProcesses;
};

Q_DECLARE_OPERATORS_FOR_FLAGS ( C_CreateFileHash::HashTypeFlag );

# endif // __KLASSE_CREATE_FILE_HASH_INCLUDED__
