/****************************************************************************
 * This file is part of the program MusicRater.                             *
 * MusicRater is a program used to rate your music while listening to them  *
 * Copyright (C) 2005, 2006 Michael "Serpedon" Walz   (serpedon@gmx.de)     *
 *                                                                          *
 * This program is free software; you can redistribute it and/or modify     *
 * it under the terms of the GNU General Public License as published by     *
 * the Free Software Foundation; either version 2 of the License, or        *
 * (at your option) any later version.                                      *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License along  *
 * with this program; if not, write to the Free Softwares Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA                 *
 ****************************************************************************/
 
# include "Klasse.MainWindow.hpp"
# include "Klasse.Config.hpp"
# include <QFile>
# include <QTimer>
# include <QMessageBox>
# include <QCloseEvent>
# include <QSettings>
# include <QTimer>
# include <QMenu>

const QString Setting_ProgPlayMusic		= "ProgPlayMusic";
const QString Setting_ProgStopMusic		= "ProgStopMusic";
const QString Setting_ProgGenerateHash	= "ProgGenerateHash";
const QString Setting_SavePointsfile	= "SavePointsfile";
const QString Setting_SaveHashfile		= "SaveHashfile";
const QString Setting_SaveMusicfile		= "SaveMusicfile";

C_MainWindow::C_MainWindow ( QWidget * parent /*= NULL*/ )
	: QMainWindow ( parent ), Ui::MainWindow ( ), m_totAmountOfPoints ( 0 ), m_curRanking ( RANK_NEUTRAL ),
		mp_curPlaying ( NULL )
{
	setupUi ( this );

	connect ( & m_musicProcess, SIGNAL ( finished ( int ) ), this, SLOT ( PlayNextTrack ( ) ) );

	connect ( lineEdit_PlayMusic,		SIGNAL ( textChanged ( QString ) ), & G_config, SLOT ( SetProgPlayMusic		( QString ) ) );
	connect ( lineEdit_StopMusic,		SIGNAL ( textChanged ( QString ) ),	& G_config, SLOT ( SetProgStopMusic		( QString ) ) );
	connect ( lineEdit_GenerateHash,	SIGNAL ( textChanged ( QString ) ), & G_config, SLOT ( SetProgGenerateHash	( QString ) ) );
	connect ( lineEdit_Pointsfile,	SIGNAL ( textChanged ( QString ) ), & G_config, SLOT ( SetSavePointsfile	( QString ) ) );
	connect ( lineEdit_Hashfile,		SIGNAL ( textChanged ( QString ) ), & G_config, SLOT ( SetSaveHashfile		( QString ) ) );
	connect ( lineEdit_Musicfile,		SIGNAL ( textChanged ( QString ) ), & G_config, SLOT ( SetSaveMusicfile		( QString ) ) );
	
	connect ( pushButton_RankWorst,		SIGNAL ( clicked ( ) ), this, SLOT ( RankWorst		( ) ) );
	connect ( pushButton_RankBad,			SIGNAL ( clicked ( ) ), this, SLOT ( RankBad			( ) ) );
	connect ( pushButton_RankNeutral,	SIGNAL ( clicked ( ) ), this, SLOT ( RankNeutral	( ) ) );
	connect ( pushButton_RankGood,		SIGNAL ( clicked ( ) ), this, SLOT ( RankGood		( ) ) );
	connect ( pushButton_RankBest,		SIGNAL ( clicked ( ) ), this, SLOT ( RankBest		( ) ) );
	
	connect ( pushButton_Next_RankWorst,	SIGNAL ( clicked ( ) ), this, SLOT ( RankWorst	( ) ) );
	connect ( pushButton_Next_RankBad,		SIGNAL ( clicked ( ) ), this, SLOT ( RankBad		( ) ) );
	connect ( pushButton_Repeat_RankGood,	SIGNAL ( clicked ( ) ), this, SLOT ( RankGood	( ) ) );
	connect ( pushButton_Repeat_RankBest,	SIGNAL ( clicked ( ) ), this, SLOT ( RankBest	( ) ) );

	connect ( pushButton_Next_RankWorst,	SIGNAL ( clicked ( ) ), this, SLOT ( Next 	( ) ) );
	connect ( pushButton_Next_RankBad,		SIGNAL ( clicked ( ) ), this, SLOT ( Next		( ) ) );
	connect ( pushButton_Repeat_RankGood,	SIGNAL ( clicked ( ) ), this, SLOT ( Repeat	( ) ) );
	connect ( pushButton_Repeat_RankBest,	SIGNAL ( clicked ( ) ), this, SLOT ( Repeat	( ) ) );
	
	LoadSavedData ( );

	QTimer::singleShot ( 1000, this, SLOT ( Start ( ) ) );
}

C_MainWindow::~C_MainWindow ( )
{
	StopMusic ( );
	
	while ( ! map_loadingMusic.isEmpty ( ) )
		delete map_loadingMusic.takeFirst ( );

	while ( ! map_approvedMusic.isEmpty ( ) )
		delete map_approvedMusic.takeFirst ( );
}

/*virtual*/ void C_MainWindow::closeEvent ( QCloseEvent * e )
{
	int answer = QMessageBox::question ( this, tr ( "MusicRater beenden?" ),
							tr ( "Soll der MusicRater wirklich beendet werden?\n"
								  "(Noch nicht gespeicherte Daten werden automatisch gespeichert.)" ),
							tr ( "&Ja, MusicRater beenden" ),
							tr ( "&Nein, MusicRater fortsetzen" ),
							QString ( ),
							1 /* No-Button */ ,
							1 /* No-Button */ );
							
	if ( answer == 0 /* Yes-Button */ )
	{
		PrepareClosingWindow ( );
		e->accept ( );
	}
	else
		e->ignore ( );
}

void C_MainWindow::PrepareClosingWindow ( )
{
	SaveUnSavedData ( );
}

void C_MainWindow::SaveUnSavedData ( )
{
	QSettings settings;
	settings.setValue ( Setting_ProgPlayMusic,	G_config.GetProgPlayMusic		( ) );
	settings.setValue ( Setting_ProgStopMusic,	G_config.GetProgStopMusic		( ) );
	settings.setValue ( Setting_ProgGenerateHash,G_config.GetProgGenerateHash	( ) );
	settings.setValue ( Setting_SavePointsfile,	G_config.GetSavePointsfile		( ) );
	settings.setValue ( Setting_SaveHashfile,		G_config.GetSaveHashfile		( ) );
	settings.setValue ( Setting_SaveMusicfile,	G_config.GetSaveMusicfile		( ) );
}

void C_MainWindow::LoadSavedData ( )
{
	QSettings settings;
	
	# define LoadStrAndSetCurToZero(f_control, f_settingValue)									\
	{																												\
		f_control->setText ( settings.value ( f_settingValue ).toString ( ) );				\
		f_control->setCursorPosition ( 0 );																\
	}
	
	LoadStrAndSetCurToZero ( lineEdit_PlayMusic,		Setting_ProgPlayMusic		);
	LoadStrAndSetCurToZero ( lineEdit_StopMusic,		Setting_ProgStopMusic		);
	LoadStrAndSetCurToZero ( lineEdit_GenerateHash,	Setting_ProgGenerateHash	);
	LoadStrAndSetCurToZero ( lineEdit_Pointsfile,	Setting_SavePointsfile		);
	LoadStrAndSetCurToZero ( lineEdit_Hashfile,		Setting_SaveHashfile			);
	LoadStrAndSetCurToZero ( lineEdit_Musicfile,		Setting_SaveMusicfile		);
	
	# undef LoadStrAndSetCurToZero
}

void C_MainWindow::PlayMusicFile ( const QString & f_filename )
{
	m_musicProcess.start ( G_config.GetProgPlayMusic ( ).arg ( f_filename ) );
}

void C_MainWindow::StopMusic ( )
{
	m_stopProcess.start ( G_config.GetProgStopMusic ( ) );
}

void C_MainWindow::Start ( )
{
	QFile file ( G_config.GetSaveMusicfile ( ) );
	if ( ! file.open ( QIODevice::ReadOnly ) )
		emit Error ( CannotOpenMusicFile );
	else
	{
		char buffer [ m_maxLenOfLine ];
		
		while ( ! file.atEnd ( ) )
		{
			qint64 amountRead = file.readLine ( buffer, m_maxLenOfLine );
			if ( amountRead > 0 )
			{
				QString filename = ( ( QString ) buffer ).trimmed ();

				if ( QFile::exists ( filename ) )
				{
					C_MusicFile * newMusicFile = new C_MusicFile ( filename );
					map_loadingMusic.append ( newMusicFile );
				}
			}
		}
	}
	
	QTimer::singleShot ( 1000, this, SLOT ( PlayNextTrack ( ) ) );
}

bool C_MainWindow::HashExists ( const QString & f_hash )
{
	for ( int counter = map_approvedMusic.size ( ) - 1; counter >= 0; counter-- )
	{
		if ( map_approvedMusic.at ( counter )->GetHash ( ) == f_hash )
			return true;
	}
	
	return false;
}

void C_MainWindow::LookAfterUnapprovedMusic ( )
{
	for ( int counter = map_loadingMusic.size ( ) - 1; counter >= 0; counter-- )
	{
		if ( map_loadingMusic.at ( counter )->IsReady ( ) )
		{
			C_MusicFile * musicFile = map_loadingMusic.takeAt ( counter );
			
			if ( ! HashExists ( musicFile->GetHash ( ) ) )
			{
				m_totAmountOfPoints += musicFile->GetPoints ( );
				m_totAmountOfPoints += C_MusicFile::GetMinAmountOfPoints ( ) * -1;
				map_approvedMusic.append ( musicFile );
			}
		}
	}
}

void C_MainWindow::PlayNextTrack ( )
{
	LookAfterUnapprovedMusic ( );
	
	int amountOfTracks = map_approvedMusic.size ( );
	
	if ( mp_curPlaying != NULL )
	{
		m_totAmountOfPoints -= mp_curPlaying->GetPoints ( );
		
		mp_curPlaying->RankFile ( m_curRanking );
		
		if ( m_curRanking == RANK_NEUTRAL && mp_curPlaying->GetPoints ( ) > 0 )
			mp_curPlaying->RankFile ( MUSIC_WAS_PLAYED_ONE_TIME );
			
		m_totAmountOfPoints += mp_curPlaying->GetPoints ( );
	}
		
	m_curRanking = RANK_NEUTRAL;
	
	if ( amountOfTracks <= 0 )
		QTimer::singleShot ( 1000, this, SLOT ( PlayNextTrack ( ) ) );
	else
	{
		if ( ! mb_repeat )
		{
			qint64 pointsIndex = rand ( ) % m_totAmountOfPoints;

			for ( int counter = map_approvedMusic.size ( ) - 1; counter >= 0; counter-- )
			{
				mp_curPlaying = map_approvedMusic.at ( counter );
				pointsIndex -= mp_curPlaying->GetPoints ( );
				pointsIndex -= C_MusicFile::GetMinAmountOfPoints ( ) * -1;
				
				if ( pointsIndex < 0 )
					break;
			}
		}
		else
			mb_repeat = false;
		
		lineEdit_Title->setText ( mp_curPlaying->GetFilename ( ) );
		lcdNumber_Points->display ( mp_curPlaying->GetPoints ( ) );
		PlayMusicFile ( mp_curPlaying->GetFilename ( ) );
	}
}
