/****************************************************************************
 * This file is part of the program MusicRater.                             *
 * MusicRater is a program used to rate your music while listening to them  *
 * Copyright (C) 2005, 2006 Michael "Serpedon" Walz   (serpedon@gmx.de)     *
 *                                                                          *
 * This program is free software; you can redistribute it and/or modify     *
 * it under the terms of the GNU General Public License as published by     *
 * the Free Software Foundation; either version 2 of the License, or        *
 * (at your option) any later version.                                      *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License along  *
 * with this program; if not, write to the Free Softwares Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA                 *
 ****************************************************************************/
 
// Schutz vor mehrfachem Einbinden
# ifndef __KLASSE_MAIN_WINDOW_HPP_INCLUDED__
# define __KLASSE_MAIN_WINDOW_HPP_INCLUDED__

# include "ui_MusicRater.h"
# include "Klasse.MusicFile.hpp"
# include <QObject>
# include <QProcess>
# include <QList>


class C_MainWindow : public QMainWindow, private Ui::MainWindow
{
	Q_OBJECT

public:
	enum MusicRank { RANK_WORST = -20, RANK_BAD = -10, MUSIC_WAS_PLAYED_ONE_TIME = 0, RANK_NEUTRAL = 0, RANK_GOOD = 10, RANK_BEST = 20 };
	
	enum ErrorReasons { CannotOpenMusicFile };

	C_MainWindow ( QWidget * parent = NULL );
	~C_MainWindow ( );

protected slots:
	void RankWorst		( ) { m_curRanking = RANK_WORST; 	};
	void RankBad 		( ) { m_curRanking = RANK_BAD; 		};
	void RankNeutral	( ) { m_curRanking = RANK_NEUTRAL;	};
	void RankGood		( ) { m_curRanking = RANK_GOOD; 		};
	void RankBest		( ) { m_curRanking = RANK_BEST; 		};
	
	void Repeat			( ) { mb_repeat = true; };
	void Next			( ) { StopMusic ( ); };
	
	void Start ( );
	
signals:
	void Error ( C_MainWindow::ErrorReasons );

protected slots:
	void PlayMusicFile ( const QString & f_filename );
	void StopMusic ( );
	void PlayNextTrack ( );
	void LookAfterUnapprovedMusic ( );

protected:
	virtual void /*QWidget::*/closeEvent ( QCloseEvent * e );

	void PrepareClosingWindow ( );
	void SaveUnSavedData ( );
	void LoadSavedData ( );
	
	bool HashExists ( const QString & f_hash );
	
	QProcess m_musicProcess;
	QProcess m_stopProcess;
	qint64 m_totAmountOfPoints;
	int m_curRanking;
	C_MusicFile * mp_curPlaying;
	bool mb_repeat;
	QList<C_MusicFile *> map_loadingMusic;
	QList<C_MusicFile *> map_approvedMusic;

	static const qint64 m_maxLenOfLine = 1024 * 64;

private:
};

# endif // __KLASSE_MAIN_WINDOW_HPP_INCLUDED__
