/****************************************************************************
 * This file is part of the program MusicRater.                             *
 * MusicRater is a program used to rate your music while listening to them  *
 * Copyright (C) 2005, 2006 Michael "Serpedon" Walz   (serpedon@gmx.de)     *
 *                                                                          *
 * This program is free software; you can redistribute it and/or modify     *
 * it under the terms of the GNU General Public License as published by     *
 * the Free Software Foundation; either version 2 of the License, or        *
 * (at your option) any later version.                                      *
 *                                                                          *
 * This program is distributed in the hope that it will be useful,          *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 * GNU General Public License for more details.                             *
 *                                                                          *
 * You should have received a copy of the GNU General Public License along  *
 * with this program; if not, write to the Free Softwares Foundation, Inc., *
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA                 *
 ****************************************************************************/
 
# include "Klasse.MusicFile.hpp"
# include "Klasse.Config.hpp"
# include "Klasse.CreateFileHash.hpp"
# include <QSettings>

C_MusicFile::C_MusicFile ( const QString & f_filename )
	: m_filename ( f_filename ), m_points ( 0 )

{
	if ( GetHash ( ).isEmpty ( ) )
	{
		C_CreateFileHash * p_createFileHash	= new C_CreateFileHash ( m_filename, true,
			/* <-- */						C_CreateFileHash::IgnoreBackID2 | C_CreateFileHash::IgnoreFrontID3 );
		
		if ( p_createFileHash )
		{
			connect ( p_createFileHash, SIGNAL ( HashCode ( QString, QString, C_CreateFileHash::HashTypeFlag ) ), this, SLOT ( SetHash ( QString ) ) );

			p_createFileHash->Start ( );
		}
	}
	else
		ReadPoints ( );
}

C_MusicFile::~C_MusicFile ( )
{
	WritePoints ( );
}

void C_MusicFile::WritePoints ( )
{
	QSettings settings ( G_config.GetSavePointsfile ( ), QSettings::IniFormat );
	settings.setValue ( GetHash ( ), m_points );
}

void C_MusicFile::RankFile ( int f_changeRank )
{
	m_points = qMax ( GetMinAmountOfPoints ( ), GetPoints ( ) + f_changeRank );
}

QString C_MusicFile::GetFilename ( )
{
	return m_filename;
}

QString C_MusicFile::GetHash ( )
{
	if ( m_hash.isEmpty ( ) )
	{
		QSettings settings ( G_config.GetSaveHashfile ( ), QSettings::IniFormat );
		m_hash = settings.value ( m_filename ).toString ( );
	}
	
	return m_hash;
}


int C_MusicFile::GetPoints ( )
{
	return m_points;
}

bool C_MusicFile::IsReady ( )
{
	return ! GetHash ( ).isEmpty ( );
}

void C_MusicFile::SetHash ( const QString & f_hash )
{
	m_hash = f_hash;
	
	QSettings settings ( G_config.GetSaveHashfile ( ), QSettings::IniFormat );
	settings.setValue ( m_filename, f_hash );
	
	ReadPoints ( );	
}

void C_MusicFile::ReadPoints ( )
{
	QSettings settings ( G_config.GetSavePointsfile ( ), QSettings::IniFormat );
	m_points = settings.value ( GetHash ( ) ).toInt ( );
}
